@@ -196,7 +196,8 @@ static int spaceball_init(int fd, unsigned long *id, unsigned long *extra)
 	if (!strncmp("Hm3003C", r, 7))
 		*id = SPACEBALL_3003C;
 
-	if (!strncmp("HvFirmware", r, 10)) {
+	/* spaceball 4000 returns 'HVFirmware' with v2.4.3 */
+	if (!strncasecmp("HvFirmware", r, 10)) {
 
 		if (spaceball_cmd(fd, "\"", r))
 			return -1;
@@ -584,6 +585,43 @@ static int dump_init(int fd, unsigned long *id, unsigned long *extra)
 		}
 }
 
+#define WACOM_IV_RESET_BAUD "\r$"
+#define WACOM_IV_RESET "\r#"
+#define WACOM_IV_STOP "SP\r"
+enum { WACOM_IV_RESET_BAUD_LEN = 2, WACOM_IV_RESET_LEN = 2, WACOM_IV_STOP_LEN = 3 };
+
+static int wacom_iv_init(int fd, unsigned long *id, unsigned long *extra)
+{
+	setline(fd, CS8 | CRTSCTS, B38400);
+	if (write(fd, WACOM_IV_RESET_BAUD, WACOM_IV_RESET_BAUD_LEN) != WACOM_IV_RESET_BAUD_LEN)
+		return -1;
+	usleep(250 * 1000);
+	if (write(fd, WACOM_IV_RESET, WACOM_IV_RESET_LEN) != WACOM_IV_RESET_LEN)
+		return -1;
+	usleep(75 * 1000);
+
+	setline(fd, CS8 | CRTSCTS, B19200);
+	if (write(fd, WACOM_IV_RESET_BAUD, WACOM_IV_RESET_BAUD_LEN) != WACOM_IV_RESET_BAUD_LEN)
+		return -1;
+	usleep(250 * 1000);
+	if (write(fd, WACOM_IV_RESET, WACOM_IV_RESET_LEN) != WACOM_IV_RESET_LEN)
+		return -1;
+	usleep(75 * 1000);
+
+	setline(fd, CS8 | CRTSCTS, B9600);
+	if (write(fd, WACOM_IV_RESET_BAUD, WACOM_IV_RESET_BAUD_LEN) != WACOM_IV_RESET_BAUD_LEN)
+		return -1;
+	usleep(250 * 1000);
+	if (write(fd, WACOM_IV_RESET, WACOM_IV_RESET_LEN) != WACOM_IV_RESET_LEN)
+		return -1;
+	usleep(75 * 1000);
+	if (write(fd, WACOM_IV_STOP, WACOM_IV_STOP_LEN) != WACOM_IV_STOP_LEN)
+		return -1;
+	usleep(30 * 1000);
+
+	return 0;
+}
+
 struct input_types {
 	const char *name;
 	const char *name2;
@@ -667,7 +705,7 @@ static struct input_types input_types[] = {
 	B2400, CS8,
 	SERIO_TWIDJOY,		0x00,	0x00,	0,	twiddler_init },
 { "--elotouch",		"-elo",		"ELO touchscreen, 10-byte mode",
-	B9600, CS8 | CRTSCTS,
+	B9600, CS8,
 	SERIO_ELO,		0x00,	0x00,	0,	NULL },
 { "--elo4002",		"-elo6b",	"ELO touchscreen, 6-byte mode",
 	B9600, CS8 | CRTSCTS,
@@ -730,6 +768,9 @@ static struct input_types input_types[] = {
 { "--w8001",		"-w8001",	"Wacom W8001",
 	B38400, CS8,
 	SERIO_W8001,		0x00,	0x00,	0,	NULL },
+{ "--wacom_iv",		"-wacom_iv",	"Wacom protocol 4 tablet",
+	B9600, CS8 | CRTSCTS,
+	SERIO_WACOM_IV,		0x00,	0x00,	0,	wacom_iv_init },
 { NULL, NULL, NULL, 0, 0, 0, 0, 0, 0, NULL }
 };
 
@@ -768,7 +809,6 @@ int main(int argc, char **argv)
 	int baud = -1;
 	int ignore_init_res = 0;
 	int no_init = 0;
-	int one_read = 0;
 
 	for (i = 1; i < argc; i++) {
 		if (!strcasecmp(argv[i], "--help")) {
@@ -884,18 +924,13 @@ int main(int argc, char **argv)
 		retval = EXIT_FAILURE;
 	}
 
+	errno = 0;
 	do {
 		i = read(fd, NULL, 0);
-		if (i == -1) {
-			if (RETRY_ERROR(errno))
-				continue;
-		} else {
-			one_read = 1;
-		}
-	} while (!i);
+	} while (RETRY_ERROR(errno));
 
 	ldisc = 0;
-	if (one_read) {
+	if (errno == 0) {
 		// If we've never managed to read, avoid resetting the line
 		// discipline - another inputattach is probably running
 		ioctl(fd, TIOCSETD, &ldisc);
